package sk.geneton.triglav.correction;

import java.util.ArrayList;
import java.util.HashMap;

import org.apache.commons.math3.analysis.interpolation.LoessInterpolator;

public class ChrGcPercentage {
	
	//private String chrName;
	/*
	 * for each percentage holds all numbers of reads that occured in a bin with given Gc content
	 */
	HashMap<Double, ArrayList<Integer>> percentageContents; 
	double[] weights;
	
	
	ChrGcPercentage(/*String chr*/){
		
		this.percentageContents = new HashMap<Double,ArrayList<Integer>>();
		for (double i = 0; i <= 100; i += 0.1){
			ArrayList<Integer> list = new ArrayList<Integer>();
			double key = (double)Math.round(i * 10) / 10;
			this.percentageContents.put(key, list);
		}
		this.weights = new double[1001];
		for (int i = 0; i < 1001; i++){
			weights[i] = 0;
		}
		//this.chrName = chr;
	}
	
	/*String getChrName(){
		return this.chrName;
	}*/
	
	void update(SamFileChContents[] sam_chromosomes, GcContents gc_c){
		
		for (int i = 0; i < sam_chromosomes.length; i++){
			
			if (sam_chromosomes[i].getBinSize() != gc_c.getBinSize()){	//expected to use the same binSize
				return;
			}
			
			String key = sam_chromosomes[i].getChrName();
			ArrayList<BinContent> list = gc_c.returnChrContents(key);
			
			/* pre kazdy bin */
			for (int j = 0; j < sam_chromosomes[i].contents.length; j++){
				int readCount = sam_chromosomes[i].contents[j];
				BinContent c = list.get(j);
				double Npercentage = c.NContent;
				double GcPercentage = c.GcContent;
				
				if (Npercentage == 0) {
					
					ArrayList<Integer> ReadNumbers = this.percentageContents.get(GcPercentage);
					if (ReadNumbers == null){
						ReadNumbers = new ArrayList<Integer>();
					}
					ReadNumbers.add(readCount);
					this.percentageContents.put(GcPercentage, ReadNumbers);
					
				}
			}
			
		}
		
		
		
	}
	
	/*
	 * returns average number of reads mapped on bins with given Gc content
	 */
	HashMap<Double, Double> average(){
		
		
		
		HashMap<Double, Double> result = new HashMap<Double,Double>();
		
		for (int k = 0; k < this.weights.length; k++){
			this.weights[k] = 0;
		}
		
		for (double i = 0; i <=100; i+=0.1){
			
			double key = (double)Math.round(i * 10) / 10;
			
			ArrayList<Integer> list = this.percentageContents.get(key);
			if (list == null){
				result.put(key, (double) 0);
			} else {
				int sum = 0;
				int n = 0;
				for (int j = 0; j < list.size(); j++){
					if (list.get(j) > 0){
						sum += list.get(j);
						n++;
					}
				}
				if (n == 0 || sum == 0) {
					result.put(key, (double) 0);
				} else {
					double avg = (double)sum/(double)n;
					result.put(key, avg);
					
					//n citani ma percentualny obsah gc i(key)
					int index =(int) Math.round(i*10);
					this.weights[index] = n; 
				}
				
			}
			
		}
		return result;
		
	}
	
	/*
	 * input is a list of averages, output is interpolated list of averages
	 */
	double[] loessInterpolation(HashMap<Double,Double> averages){
		
		
		
		LoessInterpolator interpolator = new LoessInterpolator();
		
		double[] x_values = new double[1001];
		double[] y_values = new double[1001];
		
		double j = 0;
		
		double[] result;
		for (int i = 0; i < 1001; i++){
			j = (double)Math.round(j * 10) / 10;
			x_values[i] = j;
			y_values[i] = averages.get(j);
			j+=0.1;
		}
		
		int positive_count = 0;
		for (int i = 0; i < this.weights.length; i++){
			if (this.weights[i] > 0){
				positive_count++;
			}
		}
		
		/*
		 * create arrays that include only elements with positive weihgts
		 */
		
		double[] positive_xval = new double[positive_count];
		double[] positive_yval = new double[positive_count];
		double[] positive_weights = new double[positive_count];
		
		int index = 0;
		for (int i = 0; i < this.weights.length; i++){
			if (this.weights[i] > 0){
				positive_xval[index] = x_values[i];
				positive_yval[index] = y_values[i];
				positive_weights[index] = this.weights[i];
				index++;
			}
		}
		
		//result = interpolator.smooth(x_values, y_values, this.weights);
		
		result = interpolator.smooth(positive_xval, positive_yval, positive_weights);
		
		/*
		 * transform result array back to original size
		 */

		double[] fin_result = new double[1001];
		for (int i = 0; i < fin_result.length; i++){
			fin_result[i] = 0;
		}
		
		/*for (int i = 0; i < fin_result.length; i++){
			for (int k = 0; k < positive_xval.length; k++){
			
				
			}
			
			
		}*/
		
		for (int i = 0; i < positive_xval.length; i++){
			double percentage = positive_xval[i];	//napr 54.3%
			int k = (int) Math.round(percentage*10);	//napr 543
			fin_result[k] = result[i];
		}
		
		/*double[] result = interpolator.smooth(x_values, y_values);*/
		return fin_result;
		
	}
	
	void normalizeWeights(){
		double sum = 0;
		for (int i = 0; i < this.weights.length; i++){
			sum += weights[i];
		}
		for (int i = 0; i < this.weights.length; i++){
			if (weights[i] > 0){
				weights[i] = (double)weights[i]/sum;
			}
			
		}
	}
	
}
