package sk.geneton.triglav.correction;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Scanner;

/*
 * Trieda, ktora umoznuje nacitanie GC obsahu z referencneho suboru obsahujuceho 
 * DNA sekvenciu (hg19) 
 * Pocita GC obsah v jednotlivych binoch (velkost binu ako parameter)
 * Vie vratit HashMapu GcContents s GC osahmi pre jednotlive chromozomy a biny
 * 
 *  */

public class GcContentManager {
	
	
	HashMap<String, Integer> BinCounts;
	/*
	 * nazvy chromozomov, podla ktorych sa budu vytvarat subory
	 */
	private ArrayList<String> chrNames;	
	
	/*
	 * adresa umiestnenia vyslednych binarnych suborov s lomitkom na konci, pripadne prazdny string
	 */
	private String path;
	
	/*
	 * adresa zdrojoveho suboru s DNA sekvenciou
	 */
	private String sourcePath;
	
	
	/*
	 * constructor
	 */
	GcContentManager(String _path, String _sourcePath){
		
		this.path = _path;
		this.sourcePath = _sourcePath;
	
		/*
		 * prida do chrNames chr1 - chr22, chrM, chrX, chrY
		 */
		this.chrNames = new ArrayList<String>();
		for (int i = 1; i < 23; i++){
			this.chrNames.add("chr"+i);
		}
		this.chrNames.add("chrM");
		this.chrNames.add("chrX");
		this.chrNames.add("chrY");
		
		this.BinCounts = new HashMap<String,Integer>();
	}
	
	/*
	 * get metody
	 */
	String getSourcePath(){
		return this.sourcePath;
	}
	
	String getDestPath(){
		return this.path;
	}
	
	ArrayList<String> getChrNames(){
		return this.chrNames;
	}
	
	String getChrname(int index){
		return this.chrNames.get(index);
	}
	
	/*
	 * creates binary files with name "chrN.bin" (for N-th chromosome)in folder
	 * binSize in the destination path storing number of Gc and N bases 
	 * in bins of given binSize
	 */
	void load(int binSize){
		
		/*
		 * if the Gc content was already loaded with given binSize (the file exists),
		 * stop the method
		 */
		Integer bSize = (Integer) binSize;
		
		File out = new File(this.getDestPath());
		if (!out.exists()) {
			try{
				out.mkdir();
		    } catch(SecurityException se){

		    }        
		}
		
		File theDir = new File(this.getDestPath()+bSize.toString());

		// if the directory does not exist, create it
		if (!theDir.exists()) {
			try{
				theDir.mkdir();
		    } catch(SecurityException se){

		    }        
		}
		
		File f = new File(this.getDestPath()+bSize.toString()+"/chr1.bin");
		if (f.exists()){
			return;
		}

		try {
			
			/*
			 * open the source file for reading
			 */
			Scanner scanner = new Scanner(new FileInputStream(this.getSourcePath()));
			scanner.nextLine();
			
			int chIndex = 0;
			String line = "";
			String riadok = "";
			StringBuilder sb = new StringBuilder();
			String pom;
			
			/*
			 * read till the end of the file
			 */
			while (scanner.hasNext()) {
				
				
				DataOutputStream writer = new DataOutputStream(new FileOutputStream(this.getDestPath()+binSize+"/"+this.getChrname(chIndex)+".bin"));
				line = scanner.nextLine();
				
				/*
				 * repeat until you reach next chromosome
				 */
				while (line.charAt(0) != '>') {
					
					while (line.charAt(0) != '>' && sb.length()<binSize) {
					
						sb.append(line);
						if (scanner.hasNext()){
							line = scanner.nextLine();
						} else {
							break;
						}
					}
					
					if (sb.length() > binSize) {
						riadok = sb.substring(0, binSize);
						pom = sb.substring(binSize+1);
						sb.delete(0, sb.length());
						sb.append(pom);
					} else {
						riadok = sb.toString();
						sb.delete(0, sb.length());
					}
					riadok = riadok.toUpperCase();
					
					int n = 0;
					int cg = 0;

					for (int i = 0; i < riadok.length(); i++) {
						if (riadok.charAt(i) == 'N') {
							n++;
						} else {
							if (riadok.charAt(i) == 'C'
									|| riadok.charAt(i) == 'G') {
								cg++;
							} 					
						}
					}				
					
					writer.writeInt(cg);
					writer.writeInt(n);					
					
					if (!scanner.hasNext()){
						break;
					}
				}
				
				writer.close();
				chIndex++;
			}
			scanner.close();
			
			
			
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
	}
	
	/*
	 * if the .bin files are not loaded yet, loads the files
	 * returns a hashmap storing percentage of Gc and N content 
	 * rounded to 1 decimal number
	 */
	GcContents getContents(int binSize){
		
		/*
		 * if the .bin files for given chromozome are not loaded yet, load them
		 */
		Integer bSize = binSize;
		File f = new File(this.getDestPath()+bSize.toString()+"/chr1.bin");
		if (!f.exists()){
			this.load(binSize);
		}
		
		GcContents result = new GcContents();
		result.setBinSize(binSize);
		result.contentMap = new HashMap<String,ArrayList<BinContent>>();
		
		double percentage = binSize/100;
		
		for (int i=0; i < this.chrNames.size(); i++){
			
			ArrayList<BinContent> contentValues = new ArrayList<BinContent>();
			
			int binCount = 0;
			
			try {
				
				DataInputStream in = new DataInputStream(new FileInputStream(this.getDestPath()+bSize.toString()+"/"+this.chrNames.get(i)+".bin"));	
				while (in.available() > 0){
					
					BinContent content = new BinContent();
					content.GcContent = in.readInt();					
					content.GcContent = Math.round(content.GcContent/percentage * 10) / (double)10;
					
					content.NContent = in.readInt();					
					content.NContent = Math.round(content.NContent/percentage * 10) / (double)10;
					
					contentValues.add(content);
					binCount++;
				}				
				this.BinCounts.put(this.chrNames.get(i), binCount);
				in.close();
				
			} catch (FileNotFoundException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			
			result.contentMap.put(this.chrNames.get(i), contentValues);
		}
		
		
		/*
		 * compute number of bases in each chromosome
		 */
		
		return result;
		
		
	}
	
	
}

