package sk.geneton.triglav.correction;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.PrintWriter;
import java.util.HashMap;

/*
 * This class is called with 4 arguments: input SAM file, binSize, output directory, reference to hg19.fa
 * 
 */

public class MainStatistics {
	
	public static void main(String[] args){
		
		String samFileName = args[0];
		//String samFileName = "22.sam";
		
		File samFile = new File(samFileName);
		
		//Integer binSize = 30000;
		Integer binSize = Integer.valueOf(args[1]);
		
		//String outDir = "dir";
		String outDir = args[2];
		
		//String hg = "hg19.fa";
		String hg = args[3];
		
		/*
		 * count Gc content for human DNA reference and save it in contents map
		 */
		String outdir1;
		if (!outDir.isEmpty()){
			outdir1 = outDir + "/";
		} else {
			outdir1 = outDir;
		}
		GcContentManager manager = new GcContentManager(outdir1,hg);
		GcContents contents = manager.getContents(binSize);
		/*
		 * read sam file and count number of reads for each bin
		 */
		
		SamFileChContents[] samContents = new SamFileChContents[25];
		
		/*
		 * initialize
		 */
		
		for (int i = 1; i < 23; i++){
			samContents[i-1] = new SamFileChContents("chr"+i, binSize, manager.BinCounts.get("chr"+i));
		}
		samContents[22] = new SamFileChContents("chrM", binSize, manager.BinCounts.get("chrM"));
		samContents[23] = new SamFileChContents("chrX", binSize, manager.BinCounts.get("chrX"));
		samContents[24] = new SamFileChContents("chrY", binSize, manager.BinCounts.get("chrY"));
		
		/*
		 * read Sam File
		 */
		
		for (int i = 0; i < 25; i++){
			samContents[i].readFile(samFile);
		}
		
		/*
		 * create uncorrected directory and print there values from samContents
		 */
		

		File out = new File(outDir);
		if (!out.exists()) {
			try{
				out.mkdir();
		    } catch(SecurityException se){

		    } 
		}
		File uncorrected = new File(outDir+"/uncorrected");
		if (!uncorrected.exists()) {
			try{
				uncorrected.mkdir();
		    } catch(SecurityException se){

		    } 
		}
		
		for (int i = 0; i < 22; i++){
			File f = new File(outDir+"/uncorrected/chr"+(i+1)+".txt");
			try {
				PrintWriter writer = new PrintWriter(f);
				for (int j = 0; j < samContents[i].contents.length; j++){
					writer.println(samContents[i].contents[j]);
				}
				writer.close();
				
			} catch (FileNotFoundException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} 
		}
		File fM = new File(outDir+"/uncorrected/chrM.txt");
		File fX = new File(outDir+"/uncorrected/chrX.txt");
		File fY = new File(outDir+"/uncorrected/chrY.txt");
		
		try {
			PrintWriter writer = new PrintWriter(fM);
			for (int j = 0; j < samContents[22].contents.length; j++){
				writer.println(samContents[22].contents[j]);
			}
			writer.close();
			
			writer = new PrintWriter(fX);
			for (int j = 0; j < samContents[23].contents.length; j++){
				writer.println(samContents[23].contents[j]);
			}
			writer.close();
			
			writer = new PrintWriter(fY);
			for (int j = 0; j < samContents[24].contents.length; j++){
				writer.println(samContents[24].contents[j]);
			}
			writer.close();
			
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} 
		
		/*
		 * do Gc correction
		 */
		
		/*
		 * one loess interpolation for the whole sam sample
		 */
		
		ChrGcPercentage interpolation = new ChrGcPercentage();
		interpolation.update(samContents, contents);
		HashMap<Double,Double> avg = interpolation.average();
		//interpolation.normalizeWeights();
		double[] loess = interpolation.loessInterpolation(avg);
		
		/*
		 * create corrected folder
		 */
		
		File corrected = new File(outDir+"/corrected");
		if (!corrected.exists()) {
			try{
				corrected.mkdir();
		    } catch(SecurityException se){

		    } 
		}
		
		//first 22 chromosomes
		
		for (int i = 0; i < 22; i++){
			
			CorrectedReadCounts cor = new CorrectedReadCounts(samContents[i].contents, loess, samContents[i].getReadCount(), samContents[i].notZeroReadsBinCount(), contents.contentMap.get("chr"+(i+1)));
			cor.correct();
			
			File f = new File(outDir+"/corrected/chr"+(i+1)+".txt");
			
			try {
				PrintWriter writer = new PrintWriter(f);				
				for (int j = 0; j < cor.corrected.length; j++){
					writer.println(cor.corrected[j]);
				}
				writer.close();
				
			} catch (FileNotFoundException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} 
		}
			
		//chr M
		
		CorrectedReadCounts cor = new CorrectedReadCounts(samContents[22].contents, loess, samContents[22].getReadCount(), samContents[22].notZeroReadsBinCount(), contents.contentMap.get("chrM"));
		cor.correct();
		
		File f = new File(outDir+"/corrected/chrM.txt");
		
		try {
			PrintWriter writer = new PrintWriter(f);				
			for (int j = 0; j < cor.corrected.length; j++){
				writer.println(cor.corrected[j]);
			}
			writer.close();
			
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} 
		
		
		//chrX
		
		CorrectedReadCounts cor1 = new CorrectedReadCounts(samContents[23].contents, loess, samContents[23].getReadCount(), samContents[23].notZeroReadsBinCount(), contents.contentMap.get("chrX"));
		cor1.correct();
		
		File f1 = new File(outDir+"/corrected/chrX.txt");
		
		try {
			PrintWriter writer = new PrintWriter(f1);				
			for (int j = 0; j < cor1.corrected.length; j++){
				writer.println(cor1.corrected[j]);
			}
			writer.close();
			
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} 
		
		//chrY
		
		CorrectedReadCounts cor2 = new CorrectedReadCounts(samContents[24].contents, loess, samContents[24].getReadCount(), samContents[24].notZeroReadsBinCount(), contents.contentMap.get("chrY"));
		cor2.correct();
		
		File f2 = new File(outDir+"/corrected/chrY.txt");
		
		try {
			PrintWriter writer = new PrintWriter(f2);				
			for (int j = 0; j < cor2.corrected.length; j++){
				writer.println(cor2.corrected[j]);
			}
			writer.close();
			
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} 
		
		/*
		 * correct readcounts in each chromosome
		 */
		
		/*ChrGcPercentage[] corrections = new ChrGcPercentage[25];
		for (int i = 0; i < 22; i++){
			corrections[i] = new ChrGcPercentage("chr"+(i+1));
		}
		corrections[22] = new ChrGcPercentage("chrM");
		corrections[23] = new ChrGcPercentage("chrX");
		corrections[24] = new ChrGcPercentage("chrY");
		
		//!!! nieco nefunguje pri update !!! vyzera, ze contents je v poriadku
		
		for (int i = 0; i < 25; i++){
			corrections[i].update(samContents[i], contents);			
		}
		
		File corrected = new File(outDir+"/corrected");
		if (!corrected.exists()) {
			try{
				corrected.mkdir();
		    } catch(SecurityException se){

		    } 
		}
		
		System.out.println("br6");
		
		for (int i = 0; i < 22; i++){
			HashMap<Double,Double> avg = corrections[i].average();
			double[] loess = corrections[i].loessInterpolation(avg);			
			CorrectedReadCounts cor = new CorrectedReadCounts(samContents[i].contents, loess, samContents[i].getReadCount(), samContents[i].notZeroReadsBinCount(), contents.contentMap.get("chr"+(i+1)));
			cor.correct();
			
			File f = new File(outDir+"/corrected/chr"+(i+1)+".txt");
			
			try {
				PrintWriter writer = new PrintWriter(f);
				
				/*
				 * toto cele mi tu chyba!!!!
				 * 
				 * loess metoda vrati skorigovanu hodnoutu pre percentualny pomer Gc 
				 * treba este dorobit metodu, ktora podla tejto hodnoty skoriguje pocty citani v chromozomoch
				 */
				
				// tu to bud treba zaokruhlit alebo najst iny sposob zapisovania doublov
				
			/*	for (int j = 0; j < cor.corrected.length; j++){
					writer.println(cor.corrected[j]);
				}
				writer.close();
				
			} catch (FileNotFoundException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} 
		}
		
		//chrM
		
		HashMap<Double,Double> avg = corrections[22].average();
		double[] loess = corrections[22].loessInterpolation(avg);
		CorrectedReadCounts cor = new CorrectedReadCounts(samContents[22].contents, loess, samContents[22].getReadCount(), samContents[22].notZeroReadsBinCount(), contents.contentMap.get("chrM"));
		cor.correct();
		File f = new File(outDir+"/corrected/chrM.txt");
		try {
			PrintWriter writer = new PrintWriter(f);
			
			// tu to bud treba zaokruhlit alebo najst iny sposob zapisovania doublov
			for (int j = 0; j < cor.corrected.length; j++){
				writer.println(cor.corrected[j]);
			}
			writer.close();
			
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} 
		
		//chrX
		
		HashMap<Double,Double> avg2 = corrections[23].average();
		double[] loess2 = corrections[23].loessInterpolation(avg2);
		CorrectedReadCounts cor2 = new CorrectedReadCounts(samContents[23].contents, loess2, samContents[23].getReadCount(), samContents[23].notZeroReadsBinCount(), contents.contentMap.get("chrX"));
		cor2.correct();
		File f2 = new File(outDir+"/corrected/chrX.txt");
		try {
			PrintWriter writer = new PrintWriter(f2);
					
			// tu to bud treba zaokruhlit alebo najst iny sposob zapisovania doublov		
			for (int j = 0; j < cor2.corrected.length; j++){
				writer.println(cor2.corrected[j]);
			}
			writer.close();
			
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} 
		
		//chrY
				
		HashMap<Double,Double> avg3 = corrections[24].average();
		double[] loess3 = corrections[24].loessInterpolation(avg3);
		CorrectedReadCounts cor3 = new CorrectedReadCounts(samContents[24].contents, loess3, samContents[24].getReadCount(), samContents[24].notZeroReadsBinCount() /*,manager.BinCounts.get("chrY")*//* , contents.contentMap.get("chrY"));
		/*cor3.correct();
		File f3 = new File(outDir+"/corrected/chrY.txt");
		try {
			PrintWriter writer = new PrintWriter(f3);
					
			// tu to bud treba zaokruhlit alebo najst iny sposob zapisovania doublov		
			for (int j = 0; j < cor3.corrected.length; j++){
				writer.println(cor3.corrected[j]);
			}
			writer.close();
					
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} 
		
		System.out.println("koniec");
	*/	
	}
		
}
