package sk.geneton.triglav.correction;

import java.io.File;

import net.sf.samtools.SAMFileReader;
import net.sf.samtools.SAMRecord;
import net.sf.samtools.SAMRecordIterator;


/*
 * reads sam file and counts amount of pair-reads mapped to 
 * bins in given chromozome 
 */
public class SamFileChContents {
	
	private String chr;
	private int binSize;
	private int readCount;
	
	int[] contents;	//na bin s cislom <index> sa namapovalo <hodnota> citani
	
	
	/*
	 * constructor
	 * 
	 * baseNumber je len priblizny pocet baz v chromozome()
	 */
	SamFileChContents(String chName, int bSize, int binCount){
		this.readCount = 0;
		this.chr = chName;
		this.binSize = bSize;
		this.contents = new int[binCount];
		for (int i = 0; i<binCount; i++){
			contents[i] = 0;
		}
	}
	
	int getReadCount(){
		return this.readCount;
	}
	
	int getBinSize(){
		return this.binSize;
	}
	
	String getChrName(){
		return this.chr;
	}
	
	void readFile(File f){
		
		SAMFileReader reader = new SAMFileReader(f);
		SAMRecordIterator records = reader.iterator();
				
		while (records.hasNext()){
			
			SAMRecord record1 = records.next();
			SAMRecord record2;
			if (records.hasNext()){
				record2 = records.next();
			} else {
				break;
			}
			
			/*
			 * !!! ak sa jedna o chr, kt. ma zaujima !!! 
			 * nazov chr sa zisti cez SAMRecord.getRefferenceName() 
			 */
			
			if (record1.getReferenceName().equals(this.chr) && record2.getReferenceName().equals(this.chr)){
				
			
			
				/*
				 * check relevance of the pair: 
				 * getMappingQuality() > 40
				 * abs(get.InferredInsertSize()) < 10000
				 */
			
				boolean condition1 = (record1.getMappingQuality() >= 40) && (record2.getMappingQuality() >= 40);
				boolean condition2 = Math.abs(record1.getInferredInsertSize()) < 10000 && Math.abs(record2.getInferredInsertSize()) < 10000;
			
				if (condition1 && condition2){
					/*
					 * pair-reads are valid
					 */
					this.readCount++;
					int start = record1.getAlignmentStart();	//citanie sa pripocita do binu, v ktorom zacina prve citanie z paru
					int bin = start/this.binSize;
					contents[bin]++;

				} 
			} 
			
		}
		
		reader.close();
	}
	
	int notZeroReadsBinCount(){
		int res = 0;
		for (int i = 0; i < contents.length; i++){
			if (contents[i] > 0){
				res++;
			}
		}
		return res;
	}
	
}

