package sk.geneton.triglav.correction;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.PrintWriter;
import java.util.Arrays;
import java.util.Scanner;

import org.apache.commons.math3.stat.descriptive.moment.StandardDeviation;

public class TrainingDistribution {
	
	double[] p21;
	double mi;		//priemer
	double sigma;	//smerodajna odchylka
	
	TrainingDistribution(double[] probabilities){
		
		this.p21 = probabilities;
		
		//priemer
		double sum = 0;
		for (int i = 0; i < this.p21.length; i++){
			sum += this.p21[i];
		}
		this.mi = sum/(double)this.p21.length;
		
		//smerodajna odchylka
		/*sum = 0;
		for (int i = 0; i < this.p21.length; i++){
			sum += Math.pow((this.p21[i] - this.sigma),2);
		}
		sum = Math.sqrt(sum/(double)this.p21.length);
		this.sigma = sum;*/
		StandardDeviation dev = new StandardDeviation();
		this.sigma = dev.evaluate(this.p21);
	}
	
	double zscore(double probability){
		
		double z_score = (probability - this.mi)/this.sigma;
		return z_score;
	}
	
	double mean(double[] array){
		double sum = 0;
		for (int i = 0; i < array.length; i++){
			sum += array[i];
		}
		double res = sum/array.length;
		return res;
	}
	
	double median(double[] array){
		
		Arrays.sort(array);
		double median;
		if (array.length % 2 == 0){
			median = ((double)array[array.length/2] + (double)array[array.length/2 - 1])/2;
		} else {
			median = (double) array[array.length/2];
		}
		return median;
	}
	
	double max(double[] array){
		double max = Double.MIN_VALUE;
		for (int i = 0; i < array.length; i++){
			if (array[i] > max){
				max = array[i]; 
			}
		}
		return max;
	}
	
	double min(double[] array){
		double min = Double.MAX_VALUE;
		for (int i = 0; i < array.length; i++){
			if (array[i] < min){
				min = array[i];
			}
		}
		return min;
	}
	
	public static void main(String[] args){
		
		String[] chr_names = {"chr1","chr2","chr3","chr4","chr5","chr6","chr7","chr8","chr9","chr10","chr11","chr12",
				"chr13","chr14","chr15","chr16","chr17","chr18","chr19","chr20","chr21","chr22"};
		
		String[] train_samples = {"trisomy-38-hr","trisomy-39-hr","trisomy-40-hr","trisomy-41-hr","trisomy-42-hr","trisomy-43-hr",
				"trisomy-44-hr","trisomy-45-hr","trisomy-47-hr","trisomy-56-hr","trisomy-57-hr","trisomy-58-hr","trisomy-59-hr",
				"trisomy-60-hr","trisomy-61-hr","trisomy-62-hr","trisomy-63-hr","trisomy-79-hr","trisomy-80-hr","trisomy-81-hr",
				"trisomy-84-hr","trisomy-88-hr","trisomy-89-hr","trisomy-91-hr","trisomy-92-hr","trisomy-93-hr","trisomy-94-hr",
				"trisomy-97-hr","trisomy-98-hr","trisomy-99-hr","trisomy-100-hr","trisomy-101-hr","trisomy-102-hr","trisomy-104-hr",
				"trisomy-105-hr","trisomy-106-hr","trisomy-107-hr","trisomy-108-hr","trisomy-109-hr","trisomy-111-hr",
				"trisomy-112-hr","trisomy-113-hr","trisomy-115-hr","trisomy-116-hr","trisomy-117-hr","trisomy-118-hr",
				"trisomy-120-hr","trisomy-144-hr","trisomy-148-hr","trisomy-10011-hr","trisomy-10018-hr","trisomy-10022-hr",
				"trisomy-10026-hr","trisomy-10027-hr","trisomy-10039-hr","trisomy-10044-hr","trisomy-10060-hr","trisomy-10062-hr",
				"trisomy-10068-hr","trisomy-10072-hr"};
		
		String[] test_normal = {"trisomy-2-hr","trisomy-5-hr","trisomy-6-hr","trisomy-7-hr","trisomy-8-hr","trisomy-9-hr",
				"trisomy-10-hr","trisomy-11-hr","trisomy-12-hr","trisomy-19-hr","trisomy-20-hr","trisomy-21-hr","trisomy-22-hr",
				"trisomy-23-hr","trisomy-24-hr","trisomy-28-hr","trisomy-32-hr","trisomy-34-hr","trisomy-35-hr",};
		
		String[] test_trisomic = {"trisomy-48-hr","trisomy-96a-hr","trisomy-103a-hr","trisomy-143a-hr","trisomy-201-hr","trisomy-208-hr",
				"trisomy-210-hr","trisomy-10001-hr","trisomy-10003-hr","trisomy-10005-hr","trisomy-10006-hr","trisomy-10017-hr",
				"trisomy-10019-hr","trisomy-10024-hr","trisomy-10046-hr","trisomy-10047-hr","trisomy-10048-hr","trisomy-10049-hr",
				"trisomy-10066-hr","trisomy-11002-hr","trisomy-15001-hr","trisomy-15003-hr","trisomy-15004-hr","trisomy-p1-hr",
				"trisomy-301-hr","trisomy-302-hr","trisomy-303-hr","trisomy-304-hr"};
		
		//for uncorrected read counts first
		
		double[] p21_uncorrected = new double[train_samples.length];
		
		//for each sample
		for (int i = 0; i < train_samples.length; i++){
			
			double read_sum = 0;
			double sum_21 = 0;
			
			//for each chromosome
			for (int j=0; j < chr_names.length; j++){
				
				File f = new File("counts_500kb_15-05-13/trenovacie/"+train_samples[i]+"/uncorrected/"+chr_names[j]+".txt");
				
				try {
					Scanner s = new Scanner(f);
					
					if (j == 20){
						
						while (s.hasNext()){
							String s_d = s.next();
							Double d = Double.parseDouble(s_d);
							read_sum += d;
							sum_21 += d;
						}
						
					} else {
					
						while (s.hasNext()){
							String s_d = s.next();
							Double d = Double.parseDouble(s_d);
							read_sum += d;
						}					
					}
					s.close();
					
				} catch (FileNotFoundException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				
			}
			p21_uncorrected[i] = sum_21/read_sum;
			
		}
		
		
		//create normal distribution
		TrainingDistribution dist = new TrainingDistribution(p21_uncorrected);
		
		
		//for corrected read count
		double[] p21_corrected = new double[train_samples.length];
		
		//for each sample
		for (int i = 0; i < train_samples.length; i++){
			
			double read_sum = 0;
			double sum_21 = 0;
			
			//for each chromosome
			for (int j=0; j < chr_names.length; j++){
				
				File f = new File("counts_500kb_15-05-13/trenovacie/"+train_samples[i]+"/corrected/"+chr_names[j]+".txt");
				
				try {
					Scanner s = new Scanner(f);
					
					if (j == 20){
						
						while (s.hasNext()){
							String s_d = s.next();
							Double d = Double.parseDouble(s_d);
							read_sum += d;
							sum_21 += d;
						}
						
					} else {
					
						while (s.hasNext()){
							String s_d = s.next();
							Double d = Double.parseDouble(s_d);
							read_sum += d;
						}					
					}
					s.close();
					
				} catch (FileNotFoundException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				
			}
			p21_corrected[i] = sum_21/read_sum;
			
		}
		
		//create normal distribution
		TrainingDistribution dist_cor = new TrainingDistribution(p21_corrected);
		
		double[] z_sc_uncor_nontris = new double[test_normal.length];
		double[] z_sc_uncor_tris = new double[test_trisomic.length];
		
		double[] z_score_cor_nontris = new double[test_normal.length];
		double[] z_score_cor_tris = new double[test_trisomic.length];
		
		//output file
		File file = new File("statistika_17_05_15.txt");
		try {
			
			PrintWriter writer = new PrintWriter(file);
			writer.println("NEKORIGOVANE");
			writer.println();
			
			//testing non-trisomic
			writer.println("z-score netrizomickych vzoriek:");
			for (int i = 0; i < test_normal.length; i++){
				
				double read_sum = 0;
				double sum_21 = 0;
				
				//for each chromosome
				for (int j=0; j < chr_names.length; j++){
					
					File f = new File("counts_500kb_15-05-13/test_netrizomicke/"+test_normal[i]+"/uncorrected/"+chr_names[j]+".txt");
					
					try {
						Scanner s = new Scanner(f);
						
						if (j == 20){
							
							while (s.hasNext()){
								String s_d = s.next();
								Double d = Double.parseDouble(s_d);
								read_sum += d;
								sum_21 += d;
							}
							
						} else {
						
							while (s.hasNext()){
								String s_d = s.next();
								Double d = Double.parseDouble(s_d);
								read_sum += d;
							}					
						}
						
						s.close();
					} catch (FileNotFoundException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
					
				}
				double p21 = sum_21/read_sum;
				double z = dist.zscore(p21);
				z_sc_uncor_nontris[i] = z;
				if (z > 3){
					System.out.println("trizomicka - zle");
				} else {
					System.out.println("okay");
				}
				writer.println(z);
				
			
				
			}
			
			//testing trisomic samples
			writer.println();
			writer.println("z-score trizomickych vzoriek:");
			
			for (int i = 0; i < test_trisomic.length; i++){
				
				double read_sum = 0;
				double sum_21 = 0;
				
				//for each chromosome
				for (int j=0; j < chr_names.length; j++){
					
					File f = new File("counts_500kb_15-05-13/test_trizomicke/"+test_trisomic[i]+"/uncorrected/"+chr_names[j]+".txt");
					
					try {
						Scanner s = new Scanner(f);
						
						if (j == 20){
							
							while (s.hasNext()){
								String s_d = s.next();
								Double d = Double.parseDouble(s_d);
								read_sum += d;
								sum_21 += d;
							}
							
						} else {
						
							while (s.hasNext()){
								String s_d = s.next();
								Double d = Double.parseDouble(s_d);
								read_sum += d;
							}
						
						}
						
						s.close();
						
					} catch (FileNotFoundException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
					
				}
				double p21 = sum_21/read_sum;
				double z = dist.zscore(p21);
				z_sc_uncor_tris[i] = z;
				if (z > 3){
					System.out.println("trizomicka - ok");
				} else {
					System.out.println("netrizomicka - chyba");
				}
				writer.println(z);
				
			}
			
			
			
			//corrected read counts
			writer.println();
			writer.println("KORIGOVANE");
			writer.println();
			
			//testing non-trisomic
			writer.println("z-score netrizomickych vzoriek:");
			for (int i = 0; i < test_normal.length; i++){
				
				double read_sum = 0;
				double sum_21 = 0;
				
				//for each chromosome
				for (int j=0; j < chr_names.length; j++){
					
					File f = new File("counts_500kb_15-05-13/test_netrizomicke/"+test_normal[i]+"/corrected/"+chr_names[j]+".txt");
					
					try {
						Scanner s = new Scanner(f);
						
						if (j == 20){
							
							while (s.hasNext()){
								String s_d = s.next();
								Double d = Double.parseDouble(s_d);
								read_sum += d;
								sum_21 += d;
							}
							
						} else {
						
							while (s.hasNext()){
								String s_d = s.next();
								Double d = Double.parseDouble(s_d);
								read_sum += d;
							}					
						}
						
						s.close();
					} catch (FileNotFoundException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
					
				}
				double p21 = sum_21/read_sum;
				double z = dist_cor.zscore(p21);
				z_score_cor_nontris[i] = z;
				if (z > 3){
					System.out.println("trizomicka - zle");
				} else {
					System.out.println("okay");
				}
				writer.println(z);
				
			
				
			}
			
			//testing trisomic samples
			writer.println();
			writer.println("z-score trizomickych vzoriek:");
			
			for (int i = 0; i < test_trisomic.length; i++){
				
				double read_sum = 0;
				double sum_21 = 0;
				
				//for each chromosome
				for (int j=0; j < chr_names.length; j++){
					
					File f = new File("counts_500kb_15-05-13/test_trizomicke/"+test_trisomic[i]+"/corrected/"+chr_names[j]+".txt");
					
					try {
						Scanner s = new Scanner(f);
						
						if (j == 20){
							
							while (s.hasNext()){
								String s_d = s.next();
								Double d = Double.parseDouble(s_d);
								read_sum += d;
								sum_21 += d;
							}
							
						} else {
						
							while (s.hasNext()){
								String s_d = s.next();
								Double d = Double.parseDouble(s_d);
								read_sum += d;
							}
						
						}
						
						s.close();
						
					} catch (FileNotFoundException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
					
				}
				double p21 = sum_21/read_sum;
				double z = dist_cor.zscore(p21);
				z_score_cor_tris[i] = z;
				if (z > 3){
					System.out.println("trizomicka - ok");
				} else {
					System.out.println("netrizomicka - chyba");
				}
				writer.println(z);
				
			}
			
			writer.println();
			writer.println("NEKORIGOVANE");
			writer.println();
			writer.println("netrizomicke:");
			writer.println("maximum "+dist.max(z_sc_uncor_nontris));
			writer.println("minimum "+dist.min(z_sc_uncor_nontris));
			writer.println("priemer "+dist.mean(z_sc_uncor_nontris));
			writer.println("median "+dist.median(z_sc_uncor_nontris));
			writer.println();
			writer.println("trizomicke:");
			writer.println("maximum "+dist.max(z_sc_uncor_tris));
			writer.println("minimum "+dist.min(z_sc_uncor_tris));
			writer.println("priemer "+dist.mean(z_sc_uncor_tris));
			writer.println("median "+dist.median(z_sc_uncor_tris));
			writer.println();
			writer.println("KORIGOVANE");
			writer.println();
			writer.println("netrizomicke:");
			writer.println("maximum "+dist_cor.max(z_score_cor_nontris));
			writer.println("minimum "+dist_cor.min(z_score_cor_nontris));
			writer.println("priemer "+dist_cor.mean(z_score_cor_nontris));
			writer.println("median "+dist_cor.median(z_score_cor_nontris));
			writer.println();
			writer.println("trizomicke:");
			writer.println("maximum "+dist_cor.max(z_score_cor_tris));
			writer.println("minimum "+dist_cor.min(z_score_cor_tris));
			writer.println("priemer "+dist_cor.mean(z_score_cor_tris));
			writer.println("median "+dist_cor.median(z_score_cor_tris));
			
			
			
			writer.close();
			
		} catch (FileNotFoundException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
		
		
		
		
		
	}
}
