package sk.geneton.triglav.correction;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Scanner;

import org.apache.commons.math3.stat.descriptive.moment.StandardDeviation;

public class TrisomyTesting {
	
	String[] train_samples = {"trisomy-38-hr","trisomy-39-hr","trisomy-40-hr","trisomy-41-hr","trisomy-42-hr","trisomy-43-hr",
			"trisomy-44-hr","trisomy-45-hr","trisomy-47-hr","trisomy-56-hr","trisomy-57-hr","trisomy-58-hr","trisomy-59-hr",
			"trisomy-60-hr","trisomy-61-hr","trisomy-62-hr","trisomy-63-hr","trisomy-79-hr","trisomy-80-hr","trisomy-81-hr",
			"trisomy-84-hr","trisomy-88-hr","trisomy-89-hr","trisomy-91-hr","trisomy-92-hr","trisomy-93-hr","trisomy-94-hr",
			"trisomy-97-hr","trisomy-98-hr","trisomy-99-hr","trisomy-100-hr","trisomy-101-hr","trisomy-102-hr","trisomy-104-hr",
			"trisomy-105-hr","trisomy-106-hr","trisomy-107-hr","trisomy-108-hr","trisomy-109-hr","trisomy-111-hr",
			"trisomy-112-hr","trisomy-113-hr","trisomy-115-hr","trisomy-116-hr","trisomy-117-hr","trisomy-118-hr",
			"trisomy-120-hr","trisomy-144-hr","trisomy-148-hr","trisomy-10011-hr","trisomy-10018-hr","trisomy-10022-hr",
			"trisomy-10026-hr","trisomy-10027-hr","trisomy-10039-hr","trisomy-10044-hr","trisomy-10060-hr","trisomy-10062-hr",
			"trisomy-10068-hr","trisomy-10072-hr"};
	
	String[] test_normal = {"trisomy-2-hr","trisomy-5-hr","trisomy-6-hr","trisomy-7-hr","trisomy-8-hr","trisomy-9-hr",
			"trisomy-10-hr","trisomy-11-hr","trisomy-12-hr","trisomy-19-hr","trisomy-20-hr","trisomy-21-hr","trisomy-22-hr",
			"trisomy-23-hr","trisomy-24-hr","trisomy-28-hr","trisomy-32-hr","trisomy-34-hr","trisomy-35-hr",};
	
	String[] test_trisomic = {"trisomy-48-hr","trisomy-96a-hr","trisomy-103a-hr","trisomy-143a-hr","trisomy-201-hr","trisomy-208-hr",
			"trisomy-210-hr","trisomy-10001-hr","trisomy-10003-hr","trisomy-10005-hr","trisomy-10006-hr","trisomy-10017-hr",
			"trisomy-10019-hr","trisomy-10024-hr","trisomy-10046-hr","trisomy-10047-hr","trisomy-10048-hr","trisomy-10049-hr",
			"trisomy-10066-hr","trisomy-11002-hr","trisomy-15001-hr","trisomy-15003-hr","trisomy-15004-hr","trisomy-p1-hr",
			"trisomy-301-hr","trisomy-302-hr","trisomy-303-hr","trisomy-304-hr"};
	
	/*
	 * corPositions are ArrayLists (one list for each bin on chr21) of positions of bins corelating with the current bin
	 */
	double[][] getZScores(String directory, String sample_name, ArrayList<ArrayList<Integer>> corPositions){
		
	
		ArrayList<Double> readsOther = new ArrayList<Double>(); 
		//na indexe <i> (reprezentujucom jeden bin) sa nachadza pocet citani namapovanych na bin
		
		ArrayList<Double> reads21 = new ArrayList<Double>();
		//na indexe <i> (reprezentujucom jeden bin) sa nachadza pocet citani namapovanych na bin
		
		//nacitanie poctov citani na biny	
		/*indexOther = 0;
		index21 = 0;*/
			
		//for each chromosome
		for (int j = 0; j < 22; j++){ //len chr1 - chr22
				
			if (j != 20){
				
				File f = new File(directory+sample_name+"/uncorrected/chr"+(j+1)+".txt");	//!! treba este upravit pre chX, chY a chM a podla toho ci corrected alebo uncorrected
				try {
					Scanner s = new Scanner(f);
					
					while (s.hasNext()){
						String line = s.next();
						double read = Double.parseDouble(line);
							
						readsOther.add(read);
						
						/*readsOther.set(indexOther, read);
						indexOther++;*/
					}
					s.close();
						
				} catch (FileNotFoundException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
						
			} else {	//ch21
					
				File f = new File(directory+sample_name+"/uncorrected/chr21.txt");
				try {
					Scanner s = new Scanner(f);
					while (s.hasNext()){
						String line = s.next();
						double read = Double.parseDouble(line);
							
						reads21.add(read);
							/*
							reads21.set(index21, tmp);				
							index21++;*/
					}
					s.close();
				} catch (FileNotFoundException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}		
			}
		}
		
		//return value
		double[][] z_scores = new double[reads21.size()][];
		
		//for each bin in chr21 count z-scores
		for (int i = 0; i < reads21.size(); i++){
			
			ArrayList<Integer> cor = corPositions.get(i); 
			double[] corelating = new double[cor.size()];
			z_scores[i] = new double[corelating.length];
			
			for (int j = 0; j < cor.size(); j++){
				int index = cor.get(j);
				
				//ak bin s nicim nekoreloval, hodnoty sa nastavia na NaN
				if (index < 0){
					corelating[j] = Double.NaN;
				} else {
					corelating[j] = readsOther.get(index);
				}
				
			}
			
			//for each corelating bin count a z-score
			for (int j = 0; j < corelating.length; j++){
				
				//compute differences from this bin to every other corelating bin
				double[] differences = new double[corelating.length-1];
				int id = 0;
				for(int k = 0; k < corelating.length; k++){
					if (k != j){
						differences[id] = corelating[j] - corelating[k];
						id++;
					}	
				}
				
				//create normal distribution - compute mean and standard deviation
				double mean = 0;
				for (int k = 0; k < differences.length; k++){
					mean += differences[k];
				}
				mean = mean / (double) differences.length;
				
				StandardDeviation dev = new StandardDeviation();
				double deviation = dev.evaluate(differences);
				
				//compute difference between bin on ch21 and current coreating bin
				double diff = reads21.get(i) - corelating[j];
				
				//z-score
				z_scores[i][j] = (diff - mean)/deviation;
				
			}
		}
		return z_scores;
	}
	
	int[] getBinCounts(int binSize){
		
		int res[] = new int[25];
		File hg = new File("hg19.fa");
		try {
			Scanner s = new Scanner(hg);
			
			int tmpLen = 0;
			int binCount = 1;
			String line = "";
			
			s.nextLine();
			int index = 0;
			
			while (s.hasNextLine()){
				line = s.nextLine();
				if (line.charAt(0) == '>'){
					res[index] = binCount;
					binCount = 1;
					tmpLen = 0;
					index++;
				} else {
					if ((tmpLen + line.length()) < binSize){
						tmpLen += line.length();
					} else {
						binCount++;
						tmpLen = line.length() - (binSize - tmpLen);
					}
				}
			}
			
			s.close();
			
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		return res;
	}
	
	public static void main(String[] args){
		
		int binSize = 100000;
		String directory = "counts_100kb_15-05-08"; //predpokladam, ze existuju subory s corrected a uncorrected poctami citani pre binSize 100 000
		
		TrisomyTesting correlation = new TrisomyTesting();
		
		int[] binCounts = correlation.getBinCounts(binSize);
		int size21 = binCounts[20];
		int sizeOther = 0;
		for (int i = 0; i < 22; i++){	//only chr1 - chr22
			if (i != 20){
				sizeOther += binCounts[i];
			}
		}
		
		ArrayList<ArrayList<Double>> readsOther = new ArrayList<ArrayList<Double>>(); 
		//na indexe <i> (reprezentujucom jeden bin) sa nachadza arraylist poctov citani, kazde citanie z inej vzorky
		
		ArrayList<ArrayList<Double>> reads21 = new ArrayList<ArrayList<Double>>();
		//na indexe <i> (reprezentujucom jeden bin) sa nachadza arraylist poctov citani, kazde citanie z inej vzorky
		
		//initialize
		for (int  i = 0; i < sizeOther; i++){
			readsOther.add(new ArrayList<Double>());
		}
		
		//initialize
		for (int i = 0; i < size21; i++){
			reads21.add(new ArrayList<Double>());
		}
		
		//for each train sample
		int indexOther;
		int index21;
		
		//nacitanie poctov citani na biny
		for (int i = 0; i < correlation.train_samples.length; i++){
			
			indexOther = 0;
			index21 = 0;
			
			//for each chromosome
			for (int j = 0; j < 22; j++){ //len chr1 - chr22
				
				if (j != 20){
					
					File f = new File(directory+"/trenovacie/"+correlation.train_samples[i]+"/uncorrected/chr"+(j+1)+".txt");	//!! treba este upravit pre chX, chY a chM a podla toho ci corrected alebo uncorrected
					try {
						Scanner s = new Scanner(f);
						
						while (s.hasNext()){
							String line = s.next();
							double read = Double.parseDouble(line);
							
							ArrayList<Double> temp = readsOther.get(indexOther);
							if (temp == null){
								temp = new ArrayList<Double>();
							}
							temp.add(read);
							readsOther.set(indexOther, temp);
							indexOther++;
						}
						s.close();
						
					} catch (FileNotFoundException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
					
					
				} else {	//ch21
					
					File f = new File(directory+"/trenovacie/"+correlation.train_samples[i]+"/uncorrected/chr21.txt");
					try {
						Scanner s = new Scanner(f);
						while (s.hasNext()){
							String line = s.next();
							double read = Double.parseDouble(line);
							
							ArrayList<Double> tmp = reads21.get(index21);
							if (tmp == null){
								tmp = new ArrayList<Double>();
							}
							tmp.add(read);
							reads21.set(index21, tmp);				
							index21++;
						}
						s.close();
					} catch (FileNotFoundException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
					
				}
				
				
			}
			
		}
		
		ArrayList<ArrayList<CorelationStruct>> corelate = new ArrayList<ArrayList<CorelationStruct>>();
		//on index <i> (representing i-th bin in ch21) is arraylist of bins from other that coorelate, each bin is represented by its position (index in readsOther)
		//kolko korelujucich vzoriek chceme
		
		for (int i = 0; i < size21; i++){
			corelate.add(new ArrayList<CorelationStruct>());
		}
		
		//for each bin in 21 compute correlation index with each bin in other
		for (int i = 0; i < size21; i++){
			
			ArrayList<Double> b21 = reads21.get(i);
			double[] bin21 = new double[b21.size()];
			for (int k = 0; k < bin21.length; k++){
				bin21[k] = b21.get(k);
			}
			
			
			for (int j = 0; j < sizeOther; j++){
				
				ArrayList<Double> bOther = readsOther.get(j);
				double SqEuclidianDist = 0;
				
				for (int k = 0; k < correlation.train_samples.length; k++){
					SqEuclidianDist += ((b21.get(k) - bOther.get(k)) * (b21.get(k) - bOther.get(k)));	 
				}
				if (corelate.get(i).size() < 50){
					CorelationStruct struct = new CorelationStruct();
					struct.position = j;
					struct.value = SqEuclidianDist;
					ArrayList<CorelationStruct> tmp = corelate.get(i);
					if (tmp == null){
						tmp = new ArrayList<CorelationStruct>();
					}
					tmp.add(struct);
					corelate.set(i, tmp);
					
				} else { //treba zistit, ci je mensia ako vsetky zlozky
					
					
					ArrayList<CorelationStruct> distances = corelate.get(i);
					double maximal_dist = 0;
					int max_index = 0;
					
					//najdem maximalnu vzdialenst
					for (int k = 0; k < distances.size(); k++){
						if (distances.get(k).value > maximal_dist){
							maximal_dist = distances.get(k).value;
							max_index = k;
						}
					}
					
					if (SqEuclidianDist < maximal_dist){
						CorelationStruct struct = new CorelationStruct();
						struct.position = j;
						struct.value = SqEuclidianDist;
						
						distances.set(max_index, struct);
						corelate.set(i, distances);
					}
				}
				//compute correlation oeficient
				
				/*PearsonsCorrelation corr = new PearsonsCorrelation();
				
				ArrayList<Double> bOther = readsOther.get(j);
				double[] binOther = new double[bOther.size()];
				for (int k = 0; k < binOther.length; k++){
					binOther[k] = bOther.get(k);
				}
				
				double coef = corr.correlation(bin21, binOther);
				
				if (coef >= 0.95){
					ArrayList<Integer> tmp = corelate.get(i);
					if (tmp == null){
						tmp = new ArrayList<Integer>();
					}
					tmp.add(j);
					corelate.set(i, tmp);
				}*/
				
			}
			
			//ak s binom koreluje menej ako 100 binov, tak ho chceme odignorovat
			/*if (corelate.get(i).size() < 100){
				corelate.set(i, null);
			}*/
			
			ArrayList<CorelationStruct> list = corelate.get(i);
			double sum = 0;
			for (int j = 0; j < list.size(); j++){
				sum += list.get(j).value;
			}
			
			
			if (i < 252){
				System.out.print(i+" - "+sum+": ");
				for (int j = 0; j < list.size(); j++){
					System.out.print(list.get(j).position+", ");
				}
				System.out.println();
			}
			
			
			//bin, na ktory sa ani v jednej vzorkenic nenamapovalo (pravdapodobne tam je usk N)
			/*if (sum == 0){
				
				//hodnoty sa nastavia na -1 (s tymto binom nema nic korelovat)
				for (int j = 0; j < list.size(); j++){
					CorelationStruct cs = new CorelationStruct();
					cs.position = -1;
					cs.value = -1;
					list.set(j, cs);
				}
			}*/
			
		}
		
		//copy positions of correlating bins to arrayList of arrayLists of integers
		
		ArrayList<ArrayList<Integer>> positions = new ArrayList<ArrayList<Integer>>(); 
		for (int i = 0; i < corelate.size(); i++){
			
			ArrayList<CorelationStruct> corelations = corelate.get(i);
			ArrayList<Integer> pos  = new ArrayList<Integer>();
			
			for (int j = 0; j < corelations.size(); j++){
				int p = corelations.get(j).position;
				pos.add(p);
			}
			positions.add(pos);
		}
		
		/*for (int i = 0; i < positions.size(); i++){
			ArrayList<Integer> list = positions.get(i);
			System.out.print(list.size()+": ");
			for (int j = 0; j < list.size(); j++){
				System.out.print(list.get(j)+", ");
			}
			System.out.println();
		}*/
		
		//create directory path for non-trisomic samples
		
		/*File nonTris = new File("nontrisomic");
		if (!nonTris.exists()) {
			try{
				nonTris.mkdir();
		    } catch(SecurityException se){

		    } 
		}
		
		File uncorrected = new File("nontrisomic/uncorrected");
		if (!uncorrected.exists()) {
			try{
				uncorrected.mkdir();
		    } catch(SecurityException se){

		    } 
		}
		
		//compute z-scores of non-trisomic samples
		
		for (int i = 0; i < correlation.test_normal.length; i++){
			double[][] res = correlation.getZScores(directory, "/test_netrizomicke/"+correlation.test_normal[i], positions);
			File f = new File(uncorrected+"/"+correlation.test_normal[i]+".txt");
			try {
				PrintWriter writer = new PrintWriter(f);
				for (int j = 0; j < res.length; j++){
					for (int k = 0; k < res[j].length; k++){
						writer.print(res[j][k]+" ");
					}
					writer.println();
				}
				writer.close();
				
			} catch (FileNotFoundException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		
		//create directory path for trisomic samples
		
		File tris = new File("trisomic");
		if (!tris.exists()) {
			try{
				tris.mkdir();
		    } catch(SecurityException se){

		    } 
		}
		
		File uncorr = new File("trisomic/uncorrected");
		if (!uncorr.exists()) {
			try{
				uncorr.mkdir();
		    } catch(SecurityException se){

		    } 
		}
		
		//compute z-scores of trisomic samples
		
		for (int i = 0; i < correlation.test_trisomic.length; i++){
			double[][] res = correlation.getZScores(directory, "/test_trizomicke/"+correlation.test_trisomic[i], positions);
			File f = new File(uncorr+"/"+correlation.test_trisomic[i]+".txt");
			try {
				PrintWriter writer = new PrintWriter(f);
				for (int j = 0; j < res.length; j++){
					for (int k = 0; k < res[j].length; k++){
						writer.print(res[j][k]+" ");
					}
					writer.println();
				}
				writer.close();
				
			} catch (FileNotFoundException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}*/
		
		double[][] res = correlation.getZScores(directory, "/test_trizomicke/trisomy-10001-hr", positions);
		
	}
	
}
