package sk.geneton.triglav.correction;

import static org.junit.Assert.*;

import java.io.File;

import junit.framework.Assert;

import org.junit.Before;
import org.junit.Test;

public class GcContentManagerTest {

	GcContentManager manager;
	
	@Before
	public void initialize(){
		manager = new GcContentManager("","hg19.txt");
	}
	 
	/*
	 * Testing correct initialization of GcContentManager
	 */
	
	@Test
	public void testSourcePath() {
		Assert.assertEquals(manager.getSourcePath(), "hg19.txt");
	}
	
	@Test
	public void testDestPath() {
		Assert.assertEquals(manager.getDestPath(), "");
	}
	
	@Test
	public void testFirstChr() {
		Assert.assertEquals(manager.getChrname(0), "chr1");
	}
	
	@Test
	public void test22Chr() {
		Assert.assertEquals(manager.getChrname(21), "chr22");
	}
	
	@Test
	public void testChrX() {
		Assert.assertEquals(manager.getChrname(22), "chrX");
	}
	
	@Test
	public void testChrY() {
		Assert.assertEquals(manager.getChrname(23), "chrY");
	}
	
	@Test
	public void testChrNum() {
		Assert.assertEquals(manager.getChrNames().size(), 24);
	}

	
	/*
	 * testing load method
	 */
	@Test
	public void testFileExistence() {
		manager.load(1000);
		File theDir = new File("1000");
		Assert.assertTrue(theDir.exists());
		
		File theFile = new File("1000/chr1.bin");
		Assert.assertTrue(theFile.exists());
		
		theFile = new File("1000/chr22.bin");
		Assert.assertTrue(theFile.exists());
		
		theFile = new File("1000/chrX.bin");
		assertTrue(theFile.exists());
		
		theFile = new File("1000/chrY.bin");
		assertTrue(theFile.exists());	
	}
	
	/*
	 * testing getContents method
	 */
	@Test
	public void getTest(){
		GcContents contents = manager.getContents(1000);
		
		Assert.assertNotNull(contents);
		Assert.assertNotNull(contents.contentMap);
		
		Assert.assertTrue(contents.contentMap.containsKey("chr1"));
		Assert.assertTrue(contents.contentMap.containsKey("chr22"));
		Assert.assertTrue(contents.contentMap.containsKey("chrX"));
		Assert.assertTrue(contents.contentMap.containsKey("chrY"));
		
		Assert.assertNotNull(contents.contentMap.get("chr1"));
		Assert.assertNotNull(contents.contentMap.get("chr22"));
		Assert.assertNotNull(contents.contentMap.get("chrX"));
		Assert.assertNotNull(contents.contentMap.get("chrY"));
		
		Assert.assertTrue(!contents.contentMap.get("chr1").isEmpty());
		Assert.assertTrue(!contents.contentMap.get("chr22").isEmpty());
		Assert.assertTrue(!contents.contentMap.get("chrX").isEmpty());
		Assert.assertTrue(!contents.contentMap.get("chrY").isEmpty());
	}
}
